local _G = getfenv(0);
local LibStub = _G["LibStub"];
local AddOn = LibStub("AceAddon-3.0"):GetAddon("EquivalencePoints");
local L = LibStub("AceLocale-3.0"):GetLocale("EquivalencePoints")

local format, len = _G.string.format, _G.string.len;
local gsub, trim = _G.string.gsub, _G.strtrim;

AddOn.newsCurrent = 2; -- Can only ever increase
function AddOn:News(override)
	if (not override and ((self.db.char.LastNews or 0) >= self.newsCurrent)) then return; end
	self:CreateNewsFrame();
	
	self.newsFrame:Clear();
	self.newsFrame.title:SetText(L["EQUIVALENCE POINTS"]);
	self.newsFrame.disclaimer:SetText(L["_news Disclaimer"]);
	
	-------- max length for "subject    ","body"---------------------------------------------------
	self.newsFrame:AddLine("Tune In     ","New gem rip from wowdb.com");
	self.newsFrame:AddLine("            ","");
	-------- max length for "subject    ","body"---------------------------------------------------
	self.newsFrame:AddLine("Turn on     ","A first release of this AddOn, it does pretty much what");
	self.newsFrame:AddLine("            ","EquivlancePoints part of Enhancer (my Shaman AddOn)");
	self.newsFrame:AddLine("            ","did but some extras like listing Gems (optional) in the");
	self.newsFrame:AddLine("            ","tooltip. Also allows for saving multiple set and sending");
	self.newsFrame:AddLine("            ","them in-game to others.");
	self.newsFrame:AddLine("            ","");
	-------- max length for "subject    ","body"---------------------------------------------------
	self.newsFrame:AddLine("Burn out    ","All old versions are available at the url listed as");
	self.newsFrame:AddLine("            ","website");
	self.newsFrame:AddLine("            ","");
	-------- max length for "subject    ","body"---------------------------------------------------
	self.newsFrame:AddLine("            ",""); -- Some spacing
	self.newsFrame:AddLine("            ","");
	
	self.newsFrame:AddLine(L["Command"], format(L["'/cmd config' or '/cmd config'"], L["/EqP"], L["_cmd _name Config"], L["/EquivalencePoints"], L["_cmd _name Config"]));
	self.newsFrame:AddLine(L["Website"], self:SpamSafe(L["__URL__"]));
	self.newsFrame:AddLine(L["Tracker"], self:SpamSafe(L["__EURL__"]));
	self.newsFrame:AddLine(L["E-mail"], self:SpamSafe(L["__EMAIL__"]));
	
	self.newsFrame:Show();
	self.db.char.LastNews = AddOn.newsCurrent;
end

function AddOn:SpamSafe(text)
	-- replace ' [dot] ' with '.' and ' [at] ' with '@'
	return gsub(gsub(text, " %[at%] ", "@"), " %[dot%] ", ".");
end

function AddOn:CreateNewsFrame()
	AddOn.newsFrame = CreateFrame("Frame", "AddOnNewsFrame", UIParent, "DialogBoxFrame")
	AddOn.newsFrame:SetWidth(500)
	AddOn.newsFrame:SetHeight(400)
	AddOn.newsFrame:SetPoint("CENTER")
	AddOn.newsFrame:SetBackdrop({
		bgFile = [[Interface\DialogFrame\UI-DialogBox-Background]],
	    edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]],
	    tile = true, tileSize = 16, edgeSize = 16,
	    insets = { left = 5, right = 5, top = 5, bottom = 5 }
	})
	AddOn.newsFrame:SetBackdropColor(0,0,0,1)

	local text = AddOn.newsFrame:CreateFontString(nil, "OVERLAY", "GameFontHighlightLarge")
	text:SetFont([[Interface\AddOns\EquivalencePoints\fonts\CAS_ANTN.TTF]], 24, "OUTLINE"); -- http://wdnaddons.com/2307561/FrameXML/Fonts.xml
	AddOn.newsFrame.title = text
	text:SetPoint("TOP", 0, -5)

	AddOn.newsFrame:Hide()

	AddOn.newsFrame.lefts = {}
	AddOn.newsFrame.rights = {}
	AddOn.newsFrame.textLefts = {}
	AddOn.newsFrame.textRights = {}
	function AddOn.newsFrame:Clear()
		self.title:SetText("")
		self.disclaimer:SetText("")
		for i = 1, #self.lefts do
			self.lefts[i] = nil
			self.rights[i] = nil
		end
	end
	
	local function rightColor(right)
		-- Email color as link
		right = gsub(right, "[A-Za-z0-9%.%%%+%-]+@[A-Za-z0-9%.%%%+%-]+%.%w%w%w?%w?", function(v) return "|cff007fff"..v.."|r"; end)
		
		-- URL color as link (is crap need better URL matching)
		right = gsub(right, "http%:%/%/[A-Za-z0-9%.%/]+", function(v) return "|cff007fff"..v.."|r"; end)
		
		-- '-quoted color gray
		right = gsub(right, "(%')(%/%w-%s?%w-)(%')", function(v1, v2, v3) return v1.."|cffcccccc"..v2.."|r"..v3; end) --00ff7f
		
		-- *) lists
		right = gsub(right, "^(%*%)%s)(.+)", function(v1, v2) return "|cff00cccc"..v1.."|r"..v2; end)
		
		-- return the mess
		return right;
	end

	function AddOn.newsFrame:AddLine(left, right)
		AddOn.newsFrame.lefts[#AddOn.newsFrame.lefts+1] = trim(left);
		AddOn.newsFrame.rights[#AddOn.newsFrame.rights+1] = rightColor(trim(right));
	end

	local newsFrame_Show = AddOn.newsFrame.Show
	function AddOn.newsFrame:Show(...)
		local maxLeftWidth = 0
		local maxRightWidth = 0
		local textHeight = 0
		for i = 1, #self.lefts do
			if not self.textLefts[i] then
				local left = AddOn.newsFrame:CreateFontString(nil, "OVERLAY", "GameFontNormal")
				left:SetFont([[Interface\AddOns\EquivalencePoints\fonts\CAS_ANTN.TTF]], 12, "OUTLINE");
				self.textLefts[i] = left
				local right = AddOn.newsFrame:CreateFontString(nil, "OVERLAY", "GameFontHighlight")
				right:SetFont([[Interface\AddOns\EquivalencePoints\fonts\Cella.ttf]], 10, "OUTLINE");
				self.textRights[i] = right
				if i == 1 then
					left:SetPoint("TOPRIGHT", AddOn.newsFrame, "TOPLEFT", 75, -35)
				else
					left:SetPoint("TOPRIGHT", self.textLefts[i-1], "BOTTOMRIGHT", 0, -1)
				end
				right:SetPoint("LEFT", left, "RIGHT", 5, 0)
			end
			self.textLefts[i]:SetText(self.lefts[i] .. ((len(self.lefts[i]) > 0 and len(strtrim(self.lefts[i])) > 0 and ":") or " "))
			self.textRights[i]:SetText(self.rights[i])
			local leftWidth = self.textLefts[i]:GetWidth()
			local rightWidth = self.textRights[i]:GetWidth()
			textHeight = self.textLefts[i]:GetHeight()
			if maxLeftWidth < leftWidth then
				maxLeftWidth = leftWidth
			end
			if maxRightWidth < rightWidth then
				maxRightWidth = rightWidth
			end
		end
		for i = #self.lefts+1, #self.textLefts do
			self.textLefts[i]:SetText('')
			self.textRights[i]:SetText('')
		end
		local finalWidth = 85 + maxRightWidth + 10;-- + 10 + 30;
		if (finalWidth < 360) then finalWidth = 360; end
		AddOn.newsFrame:SetWidth(finalWidth)
		AddOn.newsFrame:SetHeight(#self.lefts * (textHeight + 1) + 100)

		newsFrame_Show(self, ...)
	end
	AddOn.newsFrame:Hide()
	
	local disclaimer = AddOn.newsFrame:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmall")
	disclaimer:SetFont([[Interface\AddOns\EquivalencePoints\fonts\Cella.ttf]], 8, "");
	AddOn.newsFrame.disclaimer = disclaimer
	disclaimer:SetPoint("BOTTOMRIGHT", AddOn.newsFrame:GetName(), "BOTTOMRIGHT", -5, 5)
	
	AddOn.CreateNewsFrame = function() return; end;
end